<?php

namespace apexl\Io\modules\formbuilder\includes;

use apexl\Io\modules\formbuilder\includes\formField;

class numberField extends formField
{
    public function __construct($name)
    {
        parent::__construct($name, 'input', 'number');
    }
}
