<?php

namespace apexl\Io\modules\formbuilder\interfaces;

interface formFieldInterface
{
    public function setName($name);
    public function setId($id);
    public function setType($type);
    public function setLabel($label);
    public function isRequired();
}
